@@ -71,6 +71,7 @@ enum {
 	TD_F_SCRAMBLE_BUFFERS	= 16,
 	TD_F_VER_NONE		= 32,
 	TD_F_PROFILE_OPS	= 64,
+	TD_F_COMPRESS		= 128,
 };
 
 enum {
@@ -85,6 +86,7 @@ enum {
 	FIO_RAND_SEQ_RAND_READ_OFF,
 	FIO_RAND_SEQ_RAND_WRITE_OFF,
 	FIO_RAND_SEQ_RAND_TRIM_OFF,
+	FIO_RAND_START_DELAY,
 	FIO_RAND_NR_OFFS,
 };
 
@@ -164,6 +166,10 @@ struct thread_data {
 		os_random_state_t trim_state;
 		struct frand_state __trim_state;
 	};
+	union {
+		os_random_state_t delay_state;
+		struct frand_state __delay_state;
+	};
 
 	struct frand_state buf_state;
 
@@ -257,6 +263,7 @@ struct thread_data {
 	unsigned int latency_qd_low;
 	unsigned int latency_failed;
 	uint64_t latency_ios;
+	int latency_end_run;
 
 	/*
 	 * read/write mixed workload state
@@ -343,10 +350,10 @@ enum {
 
 #define __td_verror(td, err, msg, func)					\
 	do {								\
-		int __e = (err);						\
+		int ____e = (err);						\
 		if ((td)->error)					\
 			break;						\
-		(td)->error = __e;					\
+		(td)->error = ____e;					\
 		if (!(td)->first_error)					\
 			snprintf(td->verror, sizeof(td->verror), "file:%s:%d, func=%s, error=%s", __FILE__, __LINE__, (func), (msg));		\
 	} while (0)
@@ -368,6 +375,7 @@ extern unsigned int stat_number;
 extern int shm_id;
 extern int groupid;
 extern int output_format;
+extern int append_terse_output;
 extern int temp_stall_ts;
 extern uintptr_t page_mask, page_size;
 extern int read_only;
@@ -468,6 +476,9 @@ enum {
 };
 
 extern void td_set_runstate(struct thread_data *, int);
+extern int td_bump_runstate(struct thread_data *, int);
+extern void td_restore_runstate(struct thread_data *, int);
+
 #define TERMINATE_ALL		(-1)
 extern void fio_terminate_threads(int);
 
@@ -498,6 +509,7 @@ extern int load_blktrace(struct thread_data *, const char *, int);
  */
 extern void lat_target_check(struct thread_data *);
 extern void lat_target_init(struct thread_data *);
+extern void lat_target_reset(struct thread_data *);
 
 #define for_each_td(td, i)	\
 	for ((i) = 0, (td) = &threads[0]; (i) < (int) thread_number; (i)++, (td)++)
@@ -621,4 +633,9 @@ enum {
 	FIO_RAND_GEN_LFSR,
 };
 
+enum {
+	FIO_CPUS_SHARED		= 0,
+	FIO_CPUS_SPLIT,
+};
+
 #endif
