@@ -676,6 +676,11 @@ static int fixup_options(struct thread_data *td)
 	if (td->o.rand_seed)
 		td->o.rand_repeatable = 0;
 
+	if ((td->io_ops->flags & FIO_NOEXTEND) && td->o.file_append) {
+		log_err("fio: can't append/extent with IO engine %s\n", td->io_ops->name);
+		ret = 1;
+	}
+
 	return ret;
 }
 
@@ -1933,7 +1938,7 @@ int parse_cmd_line(int argc, char *argv[], int client_type)
 		case 'L': {
 			long long val;
 
-			if (check_str_time(optarg, &val, 1)) {
+			if (check_str_time(optarg, &val, 0)) {
 				log_err("fio: failed parsing time %s\n", optarg);
 				do_exit++;
 				exit_val = 1;
